from lib.typing import *

class Formatter:
    def format(self, prog: Program):
        return "\n".join(self.format_block(prog.functions[0].body))

    def format_expr(self, expr: Expr):
        match expr:
            case IntNode(value=value):
                return str(value)
            case RealNode(value):
                return f"{float(value):g}"
            case IdentifierNode(name=name):
                return self.format_name(name)
            case BinaryOperator(operator=operator, operand1=operand1, operand2=operand2):
                mapping = {
                    "==": "\\isequal",
                    "<": "<",
                    ">": ">",
                    "<=": "\leq",
                    ">=": "\geq",
                    "||": "\\mathbin{\\mathbf{or}}",
                    "&&": "\\mathbin{\\mathbf{and}}",
                    "+": "+",
                    "-": "-",
                    "*": "*"
                }
                return self.format_expr(operand1) + " " + mapping[operator] + " " + self.format_expr(operand2)

    def format_expr_top(self, expr: Expr):
        return "$" + self.format_expr(expr) + "$"

    def format_name(self, name: str):
        return "\\textit{" + name + "}" 

    def format_block(self, block: BlockStatement):
        result = []
        for stmt in block.statements:
            match stmt:
                case AssignmentStatement(
                    name=name, 
                    expr=UnaryOperator(operator="sample", operand=DistributionNode(operator=operator, args=args))
                ):
                    dist_str = "\\scalebox{0.95}{\\textsf{" + operator + "}}"
                    args_str = ", ".join({self.format_expr(arg) for arg in args})
                    result.append("\\zi $" + self.format_name(name) + " \\sim " + dist_str + "(" + args_str + ")$")
                case AssignmentStatement(
                    name=name,
                    expr=expr
                ):
                    result.append("\\zi $" + self.format_name(name) + " = " + self.format_expr(expr) + "$")
                case IfStatement(branches=branches):
                    for i, (cond, block) in enumerate(branches):
                        if i == 0:
                            result.append("\\zi \\If " + self.format_expr_top(cond) + ": \\Do")
                            result += self.format_block(block)
                            result.append("\\End")
                        elif cond != BoolNode(True):
                            result.append("\\zi \\kw{else} \\kw{if} " + self.format_expr_top(cond) + ": \\Do")
                            result += self.format_block(block)
                            result.append("\\End")
                        else:
                            result.append("\\zi \\kw{else}: \\Do")
                            result += self.format_block(block)
                            result.append("\\End")
                case WhileStatement(guard=guard, body=body):
                    result.append("\\zi \\While " + self.format_expr_top(guard) + ": \\Do")
                    result += self.format_block(body)
                    result.append("\\End")
        return result